#include "RUABParams.h"


static const char checkTestMsg[] = "M-Task: OK\n";


static volatile uint8_t *t;
static volatile uint8_t *tgl;

static void setupMotorsParams(motorCtrl_t *mCtrl);
static void periodMotorISR(void);

static portBASE_TYPE testTaskMOp(motorCtrl_t *mCtrl, recvPkg_t *rcvPkg, sendPkg_t *sndPkg);
static portBASE_TYPE motorOp(motorCtrl_t *mCtrl, recvPkg_t *rcvPkg, sendPkg_t *sndPkg);
static portBASE_TYPE ledSetClearOp(motorCtrl_t *mCtrl, recvPkg_t *rcvPkg, sendPkg_t *sndPkg);
static portBASE_TYPE ledOscLedOp(motorCtrl_t *mCtrl, recvPkg_t *rcvPkg, sendPkg_t *sndPkg);
static portBASE_TYPE spkOp(motorCtrl_t *mCtrl, recvPkg_t *rcvPkg, sendPkg_t *sndPkg);


static const vecCmplCtrl_t VCRCmpl[] = {
	testTaskMOp,
	motorOp,
	ledSetClearOp,
	ledOscLedOp, 
	spkOp,
};


void motorTask(void *pvParameters)
{
	motorCtrl_t *mCtrl = ((motorCtrl_t *) pvParameters);
	
	setupMotorsParams(mCtrl);
	xSemaphoreGive(mCtrl->syncTask);
	for(;;)
	{
		recvPkg_t rcvPkg;
		sendPkg_t sndPkg;
		
		if( xQueueReceive(mCtrl->motorQueue, &rcvPkg, MOTOR_QUEUE_WAIT) == pdTRUE )
			VCRCmpl[rcvPkg.op - CMPL_OPCODE_OFFSET](mCtrl, &rcvPkg, &sndPkg); 

		sendSerialPkg((mCtrl->shCtrl)->syncSendSerial, &sndPkg);
											
	}
}


static void setupMotorsParams(motorCtrl_t *mCtrl)
{
	mCtrl->led.oscLed = 0;
	mCtrl->led.pre = 0;
	tgl = &(mCtrl->led.oscLed); 
	t = &(mCtrl->led.pre); 
	timer1_compi_hook_isr = periodMotorISR;

	TIMSK |= (0x1 << TICIE1);
}


static void periodMotorISR(void)
{
	static uint8_t i = 0;
	uint8_t msk;

	if(i < *t)
		i++;
	else
	{
		msk = ( (PINB ^ (*tgl)) & LOWER_LEDS_MSK );
		PORTB = ( (PINB & (~LOWER_LEDS_MSK)) | msk );

		msk = ((*tgl >> UP_POS) ^ PING);
		PORTG = ( (PING & (~LOWER_LEDS_MSK)) | msk );

		i = 0;
	}		
}


static portBASE_TYPE testTaskMOp(motorCtrl_t *mCtrl, recvPkg_t *rcvPkg, sendPkg_t *sndPkg)
{
	(void) mCtrl;

	sndPkg->code = rcvPkg->op;
	sndPkg->len = sizeof(checkTestMsg);
	sndPkg->data = (uint8_t *)checkTestMsg;

	return pdTRUE;	
}


static portBASE_TYPE motorOp(motorCtrl_t *mCtrl, recvPkg_t *rcvPkg, sendPkg_t *sndPkg)
{
	portTickType ct;
	

	OCR1A = mCtrl->pn[R_WHEEL] + rcvPkg->wheelOff[R_WHEEL];
	OCR1B = mCtrl->pn[L_WHEEL] + rcvPkg->wheelOff[L_WHEEL];

	ct = xTaskGetTickCount();
	vTaskDelayUntil(&ct, rcvPkg->delay);

	sndPkg->code = rcvPkg->op;
	sndPkg->len = MOT_LEN_DATA;
	
	return pdTRUE;
}


static portBASE_TYPE ledSetClearOp(motorCtrl_t *mCtrl, recvPkg_t *rcvPkg, sendPkg_t *sndPkg)
{
	rcvPkg->argList[SET_LEDS] &= LED_MSK;
	PORTB = ( PINB | (rcvPkg->argList[SET_LEDS] & LOWER_LEDS_MSK) );
	PORTG = ( PING | (rcvPkg->argList[SET_LEDS] >> UP_POS) );
	mCtrl->led.oscLed &= ~(rcvPkg->argList[SET_LEDS]);

	rcvPkg->argList[CLEAR_LEDS] &= LED_MSK;
	PORTB = PINB & ~(rcvPkg->argList[CLEAR_LEDS] & LOWER_LEDS_MSK);
	PORTG = PING & ~(rcvPkg->argList[CLEAR_LEDS] >> UP_POS);
	mCtrl->led.oscLed &= ~(rcvPkg->argList[CLEAR_LEDS]);

	sndPkg->code = rcvPkg->op;
	sndPkg->len = SCL_LEN_DATA;

	return pdTRUE;	
}


static portBASE_TYPE ledOscLedOp(motorCtrl_t *mCtrl, recvPkg_t *rcvPkg, sendPkg_t *sndPkg)
{
	mCtrl->led.oscLed |= rcvPkg->argList[TOG_LEDS];
	mCtrl->led.pre = rcvPkg->argList[TIME_LEDS];

	sndPkg->code = rcvPkg->op;
	sndPkg->len = TOG_LEN_DATA;
	
	return pdTRUE;
}



static portBASE_TYPE spkOp(motorCtrl_t *mCtrl, recvPkg_t *rcvPkg, sendPkg_t *sndPkg) 
{
	(void) mCtrl;

	TCCR2 = (( TCCR2 & ~SPK_MSK ) | (rcvPkg->argList[CFG_SPK] & SPK_MSK));
	OCR2 = rcvPkg->argList[TIME_SPK];

	sndPkg->code = rcvPkg->op;
	sndPkg->len = SPK_LEN_DATA;

	return pdTRUE;
}

