#include "RUABParams.h"


static portBASE_TYPE resetOp(coreCtrl_t *, uint8_t);
static portBASE_TYPE parseTestCmplOp(coreCtrl_t *, uint8_t);
static portBASE_TYPE parseMotorOp(coreCtrl_t *, uint8_t);
#define parseSetClearLedOp	parseMotorOp
#define parseToggleLedOp	parseMotorOp
#define parseSpeakerOp		parseMotorOp


static vecParse_t parseProc[] = {
	resetOp,
	parseTestCmplOp,
	parseMotorOp,
	parseSetClearLedOp,
	parseToggleLedOp,
	parseSpeakerOp,
};


void serialTask(void *pvParameters)
{
	coreCtrl_t *cCtrl = ((coreCtrl_t *) pvParameters);
	unsigned char command;

	xSemaphoreTake(cCtrl->mCtrl.syncTask, UNDF_WAIT);
	xSemaphoreTake(cCtrl->sCtrl.syncTask, UNDF_WAIT);
	for(;;)
	{
		if( xSerialGetChar(0, (signed char *) &command, SERIAL_QUEUE_WAIT) == pdTRUE )
		{
			if(command <= CMPL_OP_THRESHOLD)
				parseProc[command] (cCtrl, command);
			else
				xQueueSendToBack(cCtrl->sCtrl.sensorQueue, ((void *) &command), 0);
		}	
	}
}


static portBASE_TYPE parseMotorOp(coreCtrl_t *cCtrl, uint8_t op)
{
	recvPkg_t pkg;

	if( xSerialGetChar(0, &((signed char *) pkg.wheelOff)[L_WHEEL], SERIAL_QUEUE_WAIT) != pdTRUE )
		return pdFALSE;
	else if( xSerialGetChar(0, &((signed char *) pkg.wheelOff)[R_WHEEL], SERIAL_QUEUE_WAIT) != pdTRUE )
		return pdFALSE;
	
	pkg.op = op;
	pkg.delay = 0;
	
	return xQueueSendToBack(cCtrl->mCtrl.motorQueue, ((void *) &pkg), 0);
}


static portBASE_TYPE parseTestCmplOp(coreCtrl_t *cCtrl, uint8_t op)
{
	recvPkg_t pkg;

	pkg.op = op;
	return xQueueSendToBack(cCtrl->mCtrl.motorQueue, ((void *) &pkg), 0);
}


static portBASE_TYPE resetOp(coreCtrl_t *cCtrl, uint8_t op)
{
	(void) op;
	
	vQueueDelete( cCtrl->mCtrl.motorQueue );
	vQueueDelete( cCtrl->mCtrl.syncTask );

	vQueueDelete( cCtrl->sCtrl.sensorQueue );
	vQueueDelete( cCtrl->sCtrl.syncTask );

	vQueueDelete( cCtrl->shCtrl.syncSendSerial );
	
	vSerialClose(0);

	vTaskEndScheduler();

	asm volatile ("jmp 0x0"); 

	return pdTRUE;
}


portBASE_TYPE hex2str(const uint8_t * const num, const uint8_t len)
{
	static const char hex[] = { 
		'0', '1', '2', '3',
		'4', '5', '6', '7',
		'8', '9', 'A', 'B',
		'C', 'D', 'E', 'F', };


	for(uint8_t i = 0; i < len; i++)
	{
		if(xSerialPutChar(0, hex[ num[i] >> 4 ], NO_WAIT) != pdTRUE)
			return pdFALSE;
		else if(xSerialPutChar(0, hex[ num[i] & 0x0F ], NO_WAIT) != pdTRUE)
			return pdFALSE;
	}

	return pdTRUE;

}


portBASE_TYPE sendSerialString(xSemaphoreHandle sync, const void *buff, const uint16_t len)
{	
	if( xSemaphoreTake(sync, SERIAL_SEND_WAIT) == pdFALSE )
		return pdFALSE;
	else
	{
		uint16_t i;

		for(i = 0; i < len; ++i)
		{
			if( xSerialPutChar(0, ((signed char *) buff)[i], SERIAL_QUEUE_WAIT) != pdTRUE )
				return pdFALSE;
		}

		xSemaphoreGive(sync);

		return pdTRUE;
	}
}


portBASE_TYPE sendSerialPkg(xSemaphoreHandle sync, const sendPkg_t const *sndPkg)
{
	if( xSemaphoreTake(sync, SERIAL_SEND_WAIT) != pdTRUE )
		return pdFALSE;
	else if( xSerialPutChar(0, (signed char) sndPkg->code, SERIAL_QUEUE_WAIT) != pdTRUE )
	{
		xSemaphoreGive(sync);
		return pdFALSE;
	}
	else 
	{
		xSerialPutChar(0, (signed char) sndPkg->err, UNDF_WAIT);
		if( sndPkg->code != 0 )
		{
			uint8_t i;

			for(i = 0; i < sndPkg->len; i++)
				xSerialPutChar(0, (signed char) sndPkg->data[i], UNDF_WAIT);		
		}

		xSemaphoreGive(sync);
		return pdTRUE;
	}	
}


